# OpenACS XML-RPC Package
This package is part of ]project-open[, an open-source enterprise project management system.

For more information about ]project-open[ please see:
* [Documentation Wiki](http://www.project-open.com/en/)
* [V5.0 Download](https://sourceforge.net/projects/project-open/files/project-open/V5.0/)
* [Installation Instructions](http://www.project-open.com/en/list-installers)

About OpenACS XML-RPC Package:

<p><h3>Overview </h3><blockquote><p> XML-RPC is a protocol which allows clients to make remote procedure calls on your server. Data is transformed into a standard XML format before being transferred between the client and server. This allows software using different OS&#39;s and programming languages to interact. See <a href="http://xml-rpc.com/">XML-RPC.com</a> for more information. </blockquote><h3>Why do we need it in OpenACS? </h3><blockquote><p> Some XML-RPC protocols have become popular in the web world. The <a href="http://www.blogger.com/developers/api/1_docs/">Blogger API</a> and the <a href="http://www.xmlrpc.com/metaWeblogApi">Metaweblog API</a> allow users to manage their blogs using tools of their choice and these have become widespread enough that users expect to find this functionality in any blogging software. For this reason, it&#39;s important to provide this minimum of functionality. </blockquote><h3>User Documentation </h3><blockquote><p> There are no user-facing pages. XML-RPC client software may require the user to know the URL that is accepting XML-RPC requests, which is admin-definable (default: http://example.com/RPC2/). XML-RPC savvy users can call the XML-RPC method <code>system.listMethods</code> to see which methods the server supports. </blockquote><h3>Admin Documentation </h3><blockquote><p> The server is installed by default at /RPC2/. Administrators can change this by unmounting the package and remounting it at the desired URL. The server can be disabled or enabled via the /admin pages. <p> The XML-RPC folks have defined a standard validation suite. These tests are implemented in the Automated Testing package, so admins can test their server against this suite (locally) by running all the automated tests. They can also go to http://validator.xmlrpc.com to test their site&#39;s validity (remotely) from there. </blockquote><h3>Developer Documentation </h3><blockquote><h4>Adding XML-RPC support to your package </h4><blockquote><p> The first thing you need to do is write the methods that you want to be available via XML-RPC. They should be defined as ad_procs just as any other OpenACS procs except for 2 differences. <ol><li> They need to be able to <strong>accept arguments</strong> as sent to them from xmlrpc::decode <p> In XML-RPC, every value has a datatype. Since TCL is a weakly-typed language, we could care less about the datatype (for the most part). So for scalar values (int, boolean, string, double, dateTime.iso8601, base64), xmlrpc::decode simply sends along the value to your proc. <strong>To recap, for scalar values, you need to do nothing special.</strong> For the 2 complex types (structs and arrays), the values are sent to your proc as TCL structures - XML-RPC structs are sent TCL arrays and XML-RPC arrays are sent as TCL lists. For example, if your proc expects a struct with 3 members (name, address and phone), then this is how the beginning of your proc will look. <pre>  array set user_info $struct
  set name $user_info(name)
  set address $user_info(address)
  set phone $user_info(phone)
  </pre> Or if your proc expects an array with n integers, which it then sums, then this is how your proc will look. <pre>  foreach num $array {
      incr sum $num
  }
  </pre><li> They need to be able to <strong>return data</strong> that xmlrpc::respond will be able to translate to XML. <p> Scalar data should be returned as a 2 item list {-datatype value}. So if your proc returns an int, its last statement might be: <pre>  return [list -int $result]
  </pre> Returning complex data structures (struct, array) is a little more *ahem* complex. One of the confusing things is the terminology. As I noted above, XML-RPC arrays are equivalent to TCL lists and XML-RPC structs are equivalent to TCL arrays. The other confusing thing is that XML-RPC is strongly typed and TCL isn&#39;t, so when you&#39;re converting from TCL to XML-RPC, you need to add the datatype for each scalar value. <ul><li>Returning an array of mixed type <pre>    return [list -array [list [list -int 36] [list -string &quot;foo&quot;]]]
    </pre><li>Returning a struct (foo=22, bar=blah) <pre>    return [list -struct [list foo [list -int 22] bar [list -string blah]]]
    </pre><li>Returning the above struct using a TCL array <pre>    set my_struct(foo) [list -int 22]
    set my_struct(bar) [list -string blah]
    return [list -struct [array get my_struct]]
    </pre><li>Returning an array of structs <pre>    set user1(name) {-string &quot;George Bush&quot;}
    set user1(id) {-int 41}
    set user2(name) {-string &quot;Bill Clinton&quot;}
    set user2(id) {-int 42}
    return [list -array [list
                             [list -struct [array get user1]]
                             [list -struct [array get user2]]]]
    </pre></ul></ol><p> Once your procs are defined in packagekey/tcl/foo-procs.tcl, register them in packagekey/tcl/foo-init.tcl. The *-init.tcl files are loaded after all the *-procs.tcl files have been loaded, so xmlrpc::register_proc will be available if the xmlrpc package is installed. Make sure you add the xmlrpc package as a dependency of your package if you register any XML-RPC procs. If you don&#39;t want your package to depend on xmlrpc, you can test for the existence of the xmlrpc_procs nsv before calling xmlrpc::register_proc <p> This registers &#39;system.listMethods&#39; <pre>xmlrpc::register_proc system.listMethods
</pre></blockquote><h4>Implementation details </h4><blockquote><p> Here is the sequence of events in an XML-RPC call. See the documentation for each proc for more details. <ol><li>A POST request is made to your XML-RPC URL.<li>The <code>xmlrpc::get_content</code> proc grabs the content of the POST request. This is a bit of a hack to cover the fact that there is no <code>ns_conn content</code> proc.<li><code>xmlrpc::invoke</code> is called to process the XML request.<li>If the server is disabled, a fault is returned<li>The XML is parsed for the methodName and arguments. <code>xmlrpc::decode_value</code> decodes the XML-RPC params into TCL variables.<li><code>xmlrpc::invoke_method</code> checks to be sure the method is registered and then attempts to call the OpenACS proc<li><code>xmlrpc::invoke</code> catches any errors from this attempt and creates a XML-RPC fault to return to the client if so. If there was no error, then <code>xmlrpc::respond</code> is called to format the result as a XML-RPC response.<li><code>xmlrpc::construct</code> does the heavy work of converting the TCL results back into valid XML-RPC params<li>Finally, if no errors occur in this process, the result is returned to the client as text/xml</ol><p> More details are provided in the ad_proc documentation for each proc. </blockquote></blockquote><h3>XML-RPC client </h3><blockquote><p> This package also implements a simple XML-RPC client. Any package that needs to make XML-RPC calls can simply add a dependency to this package and then call <code>xmlrpc::remote_call</code>. As an example, the <code>system.add</code> method sums a variable number of ints. To call the <code>system.add</code> method on http://example.com/RPC2, do this: <pre>catch {xmlrpc::remote_call http://example.com/RPC2 system.add -int 4 -int 44 -int 23} result
set result ==&gt; 71
</pre><p> It&#39;s important to <strong>always</strong> <code>catch</code> outgoing XML-RPC calls. If there&#39;s an error, it will be written to the catch variable (<code>result</code> in the example above). If there&#39;s no error, then the return value will be in <code>result</code>. <p><em>Implementation detail</em>: The client needs to be able to POST requests to other servers. The util_httppost proc in acs-tcl/tcl/utilities-procs.tcl doesn&#39;t work because it doesn&#39;t let you specify the Content-Type, which needs to be text/xml, and it doesn&#39;t add Host headers, which are required if the server you&#39;re POSTing to is using virtual hosting. So, this package implemements its own HTTP POST proc (which was stolen from lars-blogger&#39;s weblogs.com XML-RPC ping). </blockquote><h3>History of XML-RPC in OpenACS </h3><blockquote><p> The first implementation of XML-RPC for AOLServer was ns_xmlrpc, whose credits state: <blockquote> Ns_xml conversion by Dave Bauer (dave at thedesignexperience.org) with help from Jerry Asher (jerry at theashergroup.com). This code is based on the original Tcl-RPC by Steve Ball with contributions by Aaron Swartz. The original Tcl-RPC uses TclXML and TclDOM to parse the XML. It works fine but since OpenACS-4 will use ns_xml I converted it. </blockquote><p> I took this version and converted it into a OpenACS service package. All of the xml procs now use the XML abstraction procs inside acs-tcl (which currently use tDOM). All the procs are in a xmlrpc:: namespace and documentation has been added. I added support for some standard XML-RPC reserved procs (system.listMethods, system.methodHelp, system.multicall). I changed the semantics slightly in one area. XML-RPC arrays were being converted to TCL arrays, with the name of each item being an integer index. I thought it made more sense to make these TCL lists (since that is what a TCL list is anyways). It makes the code more consistent and makes it easier to understand how to deal with XML-RPC datatypes. <blockquote> XML-RPC struct = TCL array.  XML-RPC array = TCL list. </blockquote></blockquote>

# Online Reference Documentation

## Procedure Files

<table cellpadding="0" cellspacing="0"><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/procs-file-view?version_id=98015&amp;path=packages/xml-rpc/tcl/system-procs.tcl">tcl/system-procs.tcl</a></b></td><td></td><td>Standard reserved nmethods http://xmlrpc.usefulinc.com/doc/reserved.html </td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/procs-file-view?version_id=98015&amp;path=packages/xml-rpc/tcl/validator-procs.tcl">tcl/validator-procs.tcl</a></b></td><td></td><td>XML-RPC Validation Test Written by, Aaron Swartz &lt;aaron@theinfo.org&gt; Enhanced by Jerry Asher &lt;jerry@theashergroup.com&gt; Edited for xml-rpc package by Vinod Kurup &lt;vinod@kurup.com&gt; contains: xml-rpc server validation implementation xml-rpc client validator test implementation </td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/procs-file-view?version_id=98015&amp;path=packages/xml-rpc/tcl/xml-rpc-procs.tcl">tcl/xml-rpc-procs.tcl</a></b></td><td></td><td>&lt;p&gt; Initially created by Dave Bauer 2001-03-30 with inspiration from Steve Ball and help from Aaron Swartz and Jerry Asher. </td></tr></table>

## Procedures

<table cellpadding="0" cellspacing="0"><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=system.add">system.add</a></b></td><td></td><td>Simple test function. </td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=system.listMethods">system.listMethods</a></b></td><td></td><td>Enumerate the methods implemented by the XML-RPC server. </td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=system.methodHelp">system.methodHelp</a></b></td><td></td><td>This method takes one parameter, the name of a method implemented by the XML-RPC server. </td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=system.multicall">system.multicall</a></b></td><td></td><td>&lt;p&gt; Perform multiple requests in one call - see http://www.xmlrpc.com/discuss/msgReader$1208 &lt;/p&gt; &lt;p&gt; Takes an array of XML-RPC calls encoded as structs of the form (in a Pythonish notation here): &lt;pre&gt; {&#39;methodName&#39;: string, &#39;params&#39;: array} &lt;/pre&gt; &lt;/p&gt; </td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=xmlrpc::enabled_p">xmlrpc::enabled_p</a></b></td><td></td><td></td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=xmlrpc::list_methods">xmlrpc::list_methods</a></b></td><td></td><td></td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=xmlrpc::register_proc">xmlrpc::register_proc</a></b></td><td></td><td>&lt;p&gt; Register a proc to be available via XML-RPC. </td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=xmlrpc::remote_call">xmlrpc::remote_call</a></b></td><td></td><td>Invoke a method on a remote server using XML-RPC </td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/proc-view?version_id=98015&amp;proc=xmlrpc::url">xmlrpc::url</a></b></td><td></td><td></td></tr></table>

## SQL Files

<table cellpadding="0" cellspacing="0"></table>

## Content Pages

<table cellpadding="0" cellspacing="0"><tr valign="top"><td><b>www/</b></td></tr><tr valign="top"><td><b>admin/</b></td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/content-page-view?version_id=98015&amp;path=packages/xml-rpc/www/admin/index.adp">index.adp</a></b></td><td></td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/content-page-view?version_id=98015&amp;path=packages/xml-rpc/www/admin/index.tcl">index.tcl</a></b></td><td>Front page of admin</td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/content-page-view?version_id=98015&amp;path=packages/xml-rpc/www/admin/toggle.tcl">toggle.tcl</a></b></td><td>Toggle the server status</td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/content-page-view?version_id=98015&amp;path=packages/xml-rpc/www/index.adp">index.adp</a></b></td><td></td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/content-page-view?version_id=98015&amp;path=packages/xml-rpc/www/index.tcl">index.tcl</a></b></td><td>Accept XML-RPC POST requests and processes them.</td></tr><tr valign="top"><td style="width:35%"><b><a href="http://www.project-open.net/api-doc/content-page-view?version_id=98015&amp;path=packages/xml-rpc/www/test_login.tcl">test_login.tcl</a></b></td><td></td></tr></table>

